import type {
  RestPathParams,
  RestQueryParams,
  RestRequestBody,
  RestRequestHeaders,
  RestRequireAuthentication,
  RestResponseBody,
} from './common'

export type RestHttpMethod = 'GET' | 'POST' | 'DELETE' | 'PATCH' | 'PUT' | 'OPTIONS'
export type RestRoutePath = `/${string}`

export type RestApiRoute<
  PathParams extends RestPathParams = never,
  QueryParams extends RestQueryParams = never,
  RequestHeaders extends RestRequestHeaders = never,
  RequestBody extends RestRequestBody = never,
  ResponseBody extends RestResponseBody = never,
  RequireAuthentication extends RestRequireAuthentication = false,
> = {
  method: RestHttpMethod
  path: RestRoutePath
  buildUrl: (pathParams: PathParams, queryParams: QueryParams) => RestRoutePath
  requireAuthentication: RequireAuthentication

  // this is a workaround to force optional generics to be typesafe
  __types?: {
    RequestHeaders: RequestHeaders
    RequestBody: RequestBody
    ResponseBody: ResponseBody
  }
}
